@@ -6,6 +6,8 @@
  *
  * License: 3-clause BSD, see https://opensource.org/licenses/BSD-3-Clause
  */
+
+use anyhow::Context;
 use indicatif::{ProgressBar, ProgressStyle};
 use slog::{crit, info};
 //use num_format::{Locale};
@@ -198,19 +200,17 @@ pub fn bam2rad(input_file: String, rad_file: String, num_threads: u32, log: &slo
             .expect("coudn't write to output file");
 
         // read-level
-        let bc_string_in;
-        if let Ok(Aux::String(bcs)) = rec.aux(b"CR") {
-            bc_string_in = bcs.to_string();
+        let bc_string_in: &str = if let Ok(Aux::String(bcs)) = rec.aux(b"CR") {
+            bcs
         } else {
-            panic!("Input record missing CR tag!");
-        }
+            panic!("Input record missing CR tag!")
+        };
 
-        let umi_string_in;
-        if let Ok(Aux::String(umis)) = rec.aux(b"UR") {
-            umi_string_in = umis.to_string();
+        let umi_string_in: &str = if let Ok(Aux::String(umis)) = rec.aux(b"UR") {
+            umis
         } else {
-            panic!("Input record missing UR tag!");
-        }
+            panic!("Input record missing UR tag!")
+        };
 
         let bclen = bc_string_in.len() as u16;
         let umilen = umi_string_in.len() as u16;
@@ -382,19 +382,17 @@ pub fn bam2rad(input_file: String, rad_file: String, num_threads: u32, log: &slo
 
         // if this is a new read update the old variables
         {
-            let bc_string_in;
-            if let Ok(Aux::String(bcs)) = rec.aux(b"CR") {
-                bc_string_in = bcs.to_string();
+            let bc_string_in: &str = if let Ok(Aux::String(bcs)) = rec.aux(b"CR") {
+                bcs
             } else {
-                panic!("Input record missing CR tag!");
-            }
+                panic!("Input record missing CR tag!")
+            };
 
-            let umi_string_in;
-            if let Ok(Aux::String(umis)) = rec.aux(b"UR") {
-                umi_string_in = umis.to_string();
+            let umi_string_in: &str = if let Ok(Aux::String(umis)) = rec.aux(b"UR") {
+                umis
             } else {
-                panic!("Input record missing UR tag!");
-            }
+                panic!("Input record missing UR tag!")
+            };
 
             let bc_string = bc_string_in.replacen('N', "A", 1);
             let umi_string = umi_string_in.replacen('N', "A", 1);
@@ -497,7 +495,7 @@ pub fn view2(
     print_header: bool,
     _out_file: String,
     log: &slog::Logger,
-) -> Result<u64, Box<dyn std::error::Error>> {
+) -> anyhow::Result<u64> {
     let i_file = File::open(rad_file).unwrap();
     let mut br = BufReader::new(i_file);
     let hdr = rad_types::RadHeader::from_bytes(&mut br);
@@ -552,9 +550,9 @@ pub fn view2(
     let mut num_reads: u64 = 0;
 
     let bc_type = rad_types::decode_int_type_tag(bct.expect("no barcode tag description present."))
-        .expect("unknown barcode type id.");
+        .context("unknown barcode type id.")?;
     let umi_type = rad_types::decode_int_type_tag(umit.expect("no umi tag description present"))
-        .expect("unknown barcode type id.");
+        .context("unknown barcode type id.")?;
 
     let stdout = stdout(); // get the global stdout entity
     let stdout_l = stdout.lock();
