@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020-2021 Rob Patro, Avi Srivastava, Hirak Sarkar, Dongze He, Mohsen Zakeri.
+ * Copyright (c) 2020-2022 Rob Patro, Avi Srivastava, Hirak Sarkar, Dongze He, Mohsen Zakeri.
  *
  * This file is part of alevin-fry
  * (see https://github.com/COMBINE-lab/alevin-fry).
@@ -9,10 +9,12 @@
 
 pub mod cellfilter;
 pub mod collate;
+pub mod constants;
 pub mod convert;
 pub mod em;
 pub mod eq_class;
 pub mod infer;
+pub mod io_utils;
 pub mod pugutils;
 pub mod quant;
 pub mod utils;
