@@ -7,17 +7,9 @@
  * License: 3-clause BSD, see https://opensource.org/licenses/BSD-3-Clause
  */
 
-extern crate bio_types;
-extern crate chrono;
-extern crate clap;
-extern crate itertools;
-extern crate num_cpus;
-extern crate rand;
-extern crate slog;
-extern crate slog_term;
-
+use anyhow::anyhow;
 use bio_types::strand::Strand;
-use clap::{arg, crate_authors, crate_version, App, AppSettings, ArgSettings};
+use clap::{arg, crate_authors, crate_version, Command};
 use csv::Error as CSVError;
 use csv::ErrorKind;
 use itertools::Itertools;
@@ -47,7 +39,7 @@ fn gen_random_kmer(k: usize) -> String {
     s
 }
 
-fn main() -> Result<(), Box<dyn std::error::Error>> {
+fn main() -> anyhow::Result<()> {
     let num_hardware_threads = num_cpus::get() as u32;
     let max_num_threads: String = (num_cpus::get() as u32).to_string();
     let max_num_collate_threads: String = (16_u32.min(num_hardware_threads).max(2_u32)).to_string();
@@ -58,7 +50,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     // capture the entire command line as a string
     let cmdline = std::env::args().join(" ");
 
-    let convert_app = App::new("convert")
+    let convert_app = Command::new("convert")
         .about("Convert a BAM file to a RAD file")
         .version(version)
         .author(crate_authors)
@@ -69,19 +61,19 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
         )
         .arg(arg!(-o --output <RADFILE> "output RAD file"));
 
-    let view_app = App::new("view")
+    let view_app = Command::new("view")
         .about("View a RAD file")
         .version(version)
         .author(crate_authors)
         .arg(arg!(-r --rad <RADFILE> "input RAD file"))
         .arg(
-            arg!(-h --header "flag for printing header")
+            arg!(-H --header "flag for printing header")
                 .takes_value(false)
                 .required(false),
         )
         .arg(arg!(-o --output <RADFILE> "output plain-text-file file").required(false));
 
-    let gen_app = App::new("generate-permit-list")
+    let gen_app = Command::new("generate-permit-list")
         .about("Generate a permit list of barcodes from a RAD file")
         .version(version)
         .author(crate_authors)
@@ -114,7 +106,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                 .required(true));
     //.arg(Arg::from("-v --velocity-mode 'flag for velocity mode'").takes_value(false).required(false));
 
-    let collate_app = App::new("collate")
+    let collate_app = Command::new("collate")
     .about("Collate a RAD file by corrected cell barcode")
     .version(version)
     .author(crate_authors)
@@ -127,7 +119,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     //.arg(arg!(-e --expected-ori=[expected-ori] 'the expected orientation of alignments'")
     //     .default_value(fw"));
 
-    let quant_app = App::new("quant")
+    let quant_app = Command::new("quant")
     .about("Quantify expression from a collated RAD file")
     .version(version)
     .author(crate_authors)
@@ -147,10 +139,11 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     .arg(arg!(--"sa-model" "preferred model of splicing ambiguity")
         .possible_values(&["prefer-ambig", "winner-take-all"])
         .default_value("winner-take-all")
-        .setting(ArgSettings::Hidden))
-    .arg(arg!(--"small-thresh" <SMALLTHRESH> "cells with fewer than these many reads will be resolved using a custom approach").default_value("10").setting(ArgSettings::Hidden));
+        .hide(true))
+    .arg(arg!(--"small-thresh" <SMALLTHRESH> "cells with fewer than these many reads will be resolved using a custom approach").default_value("10")
+        .hide(true));
 
-    let infer_app = App::new("infer")
+    let infer_app = Command::new("infer")
     .about("Perform inference on equivalence class count data")
     .version(version)
     .author(crate_authors)
@@ -163,8 +156,9 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     .arg(arg!(--"quant-subset" <SFILE> "file containing list of barcodes to quantify, those not in this list will be ignored").required(false))
     .arg(arg!(--"use-mtx" "flag for writing output matrix in matrix market instead of EDS").takes_value(false).required(false));
 
-    let opts = App::new("alevin-fry")
-        .setting(AppSettings::SubcommandRequiredElseHelp)
+    let opts = Command::new("alevin-fry")
+        .subcommand_required(true)
+        .arg_required_else_help(true)
         .version(version)
         .author(crate_authors)
         .about("Process RAD files from the command line")
@@ -191,25 +185,6 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     // You can handle information about subcommands by requesting their matches by name
     // (as below), requesting just the name used, or both at the same time
-    /*
-    if let Some(ref t) = opts.subcommand_matches("test") {
-        let input_file: String = t.value_of_t("input").expect("no input string specified");
-        let rad_dir: String = t.value_of_t("rad-dir").expect("no input string specified");
-        let min_reads: usize = t
-            .value_of_t("min-reads")
-            .expect("min-reads must be a valid integer");
-        if min_reads < 1 {
-            crit!(
-                log,
-                "min-reads < 1 is not supported, the value {} was provided",
-                min_reads
-            );
-            std::process::exit(1);
-        }
-        let _r = test_external_parse(input_file, rad_dir, min_reads, &log);
-    }
-    */
-
     if let Some(t) = opts.subcommand_matches("generate-permit-list") {
         let input_dir: String = t.value_of_t("input").expect("no input directory specified");
         let output_dir: String = t
@@ -435,7 +410,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                             match *error.kind() {
                                 // if a deserialize error, we already complained about it
                                 ErrorKind::Deserialize { .. } => {
-                                    return Err("execution terminated unexpectedly".into())
+                                    return Err(anyhow!("execution terminated unexpectedly"));
                                 }
                                 // if another type of error, just panic for now
                                 _ => {
@@ -477,7 +452,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                             match *error.kind() {
                                 // if a deserialize error, we already complained about it
                                 ErrorKind::Deserialize { .. } => {
-                                    return Err("execution terminated unexpectedly".into())
+                                    return Err(anyhow!("execution terminated unexpectedly"));
                                 }
                                 // if another type of error, just panic for now
                                 _ => {
